#! /usr/bin/perl
#
#  gettile.pl:		uh, Fri May 15 1998
#
#  Written by:		Ullrich Hafner
#  
#  Copyright (C) 1998 Ullrich Hafner <hafner@bigfoot.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
#
#  $Id: gettile.pl.in,v 1.20 1999/03/07 18:13:18 hafner Exp $
#  $Date: 1999/03/07 18:13:18 $
#  $Author: hafner $
#  $Revision: 1.20 $
#  $State: Exp $
#

################################################################################
###
###	usage : gettile.pl WindowMaker pixmap-filename wmaker-attribute [offset]
### 
###     gets pixmap via HTTP request from http://themes.org/
###     Links `pixmap-filename' to the received pixmap und updates
###     users `WindowMaker' file.
###     'pixmap-filename' must contain the pattern
###     tile-of-the-day, random-tile, guest-tile, root-of-the-day,
###     or bar-of-the-day
###
################################################################################

use File::Basename;

sub quit;

if ($#ARGV != 2 && $#ARGV != 3)
{
   quit ("usage: $0 WindowMaker pixmap-filename wmaker-attribute [offset]");
}

# get commandline parameters
$windowmaker = $ARGV[0];
$tilename    = $ARGV[1];
$attribute   = $ARGV[2];
$offset      = ($#ARGV == 3) ? $ARGV[3] : 0;

# insert configure variables
$wwwget      = "/usr/bin/wget";
$getscript   = "www";
$wdwrite     = "/usr/X11R6/bin/wdwrite";
$getstyle    = $wdwrite;
$getstyle    =~ s/wdwrite/getstyle/;
$tmpfile     = dirname ($tilename) . "/$$.gettile";

if ($getscript eq "www")
{
   if ($wwwget =~ /wget/)
   {
      $getscript = "wget.pl";
   }
   else
   {
      $getscript = "lynx.pl";
   }
}

$include = $0;
$include =~ s/gettile.pl/$getscript/;

# First of all, compute URL to get pixmap from
if ($tilename =~ /of-the-day/) # {tile,root,bar} of the day
{
   $tval  = time;
   $tval -= 60 * 60 * 30;	    # new tile at 0:00 CST - 1 day 
   $tval += 60 * 60 * 24 * $offset; # user defined offset
   
   @today = gmtime $tval;
   @month = ("Jan", "Feb", "Mar", "Apr", "May", "Jun",
	     "Jul", "Aug", "Sep", "Oct", "Nov", "Dec");
   $date  = "$month[$today[4]]$today[3]";
   if ($tilename =~ /tile-of/)
   {
      $url = "http://themes.org/refresh/tiles/$date.jpg";
   }
   else
   {
      $url = "http://themes.org/refresh/bars/$date.jpg";
   }
}
else				# random tiles for masses, or guest tile
{
   # the pixap URL is not known apriori, we must extract the URL from
   # the toplevel WWW page
   if ($tilename =~ /random-tile/)
   {
      $url = "http://themes.org/rtfm/";
   }
   elsif ($tilename =~ /guest-tile/)
   {
      $url = "http://themes.org/totd/guest/";
   }

   # get corresponding tile HTML page
   do "$include";
   quit ("$@") if ($@);

   # find text background="tilename"	random-tile
   # find text guest/archive		guest-tile
   open (RTFM, $tmpfile) || quit ("Can't open File : $!");

   $url = "";
   while (<RTFM>)
   {
      if (($tilename =~ /random-tile/)
	  && (/background=["]([^"]*)\"/))
      {
         $url = $1;
	 unless ($url =~ /http/)
	 {
	    $url = "http://themes.org/rtfm/$url";
	 }
         last;
      }
      elsif (($tilename =~ /guest-tile/)
	     && (/img.*src.*["](.*guest\/archive\/[^"]*)\"/))
      {
         $url = $1;
         last;
      }
   }
   close RTFM;
   unlink $tmpfile;
   
   # rtfm tile name
   if ($url eq "")
   {
      $url = "http://themes.org/refresh/tiles/Mar1.jpg"; # default
   }
}

# get new tile with libwww-perl or lynx
$file = "$tilename";
do "$include"; 
quit ("$@") if ($@);

$filetype = `file $tmpfile`;
quit ("Received data contains HTML code...") if ($filetype =~ /HTML/i);

rename ($tmpfile, $file) || die "Can't rename '$tmpfile' to '$file': $!";
$file = "$tilename";

# update WindowMaker file unless $windowmaker = "no"

unless (($windowmaker eq "no") || ($wdwrite eq "no"))
{
   # check whether a text color has been specified in the image file
   open (PIXMAP, "$file") || die "Can't open File $file: $!";
   $color = "";
   while (<PIXMAP>)
   {
      if (/color=\"?([a-z]+|\#[0-9a-f]{6,6})/i)
      {
	 $color = $1;
	 if ($file =~ /tile/)	# IconBack
	 {
	    $colorname = "ClipTitleColor";
	 }
	 else
	 {
	    $colorname = $attribute;
	    $colorname =~ s/Back/Color/i;
	 }
	 system ("$wdwrite WindowMaker $colorname '\"$color\"'")
	   && die "$wdwrite error.";
      }
   }
   close PIXMAP;

   # get current value of $attribute
   $c_colorname  = "#000042";
   $c_pixmaptype = "s";
   open (GETSTYLE, "$getstyle -t |")
     || die "Can't run commmand '$getstyle -t': $!";
   while (<GETSTYLE>)
   {
      if (/$attribute/i)	# found attribute value
      {
	 $line = $_;
	 chomp $line;
	 until ($line =~ /;/)	# read until we find a terminating semicolon
	 {
	    if ($part = <GETSTYLE>)
	    {
	       $line .= $part;
	       chomp $line;
	    }
	    else
	    {
	       last;
	    }
	 }
	 if ($line =~ /$attribute.*,.*,\s*\"?(\#[a-f0-9]{5,5})([a-f0-9])\"?\s*\)/i)
	 {
	    $char =  lc ($2);
	    $char =~ tr/0123456789abcdef/1032547698badcfe/;
	    $c_colorname = "$1$char";
	 }
	 if ($line =~ /$attribute.*\(\s*([a-z])pixmap\s*,.*,.*\)/i)
	 {
	    $c_pixmaptype = $1;
	 }
      }
   }
   close GETSTYLE;
   $file = substr $file, ($p + 1) unless (($p = rindex ($file, "/")) < 0);
   
   system ("$wdwrite WindowMaker $attribute '($c_pixmaptype"
	   . "pixmap, \"$file\", \"$c_colorname\")'") && die "$wdwrite error.";
}

sub quit
{
   $msg = shift;
   unlink $tmpfile if -r $tmpfile;
   die "$msg";
}
